#' Function to get a palette from the ETHZ web site
#'
#' The functions retrieves a palette from the ETHZ web site
#'    https://wiki.c2sm.ethz.ch/pub/Data/VisNCLCosmoLibraryColorTables/
#'
#' @param palette.name  Name of the palette as listed on the ETHZ web site
#' @param url url of the ETHZ palette web site
#'
#' @return A list variable with
#' \item{colors}{The colours of the palette}
#' \item{levels}{An example of levels}
#' 
#' @author Burkhardt Rockel (HZG)
#' 
#' @export

get_ethz_palette <-
  function(palette.name, url="https://wiki.c2sm.ethz.ch/pub/Data/VisNCLCosmoLibraryColorTables/")
  {
    file<-paste(url,palette.name,".ct",sep="")
    urlcolors<-readLines(file)
    urlcolors<-urlcolors[!startsWith(urlcolors,';')]
    n.lines<-length(urlcolors)
    n.levels<-as.numeric(urlcolors[1])
    levels<-as.numeric(scan(text=urlcolors[2], what="",quiet=TRUE))
    colors<-c(1:(n.levels+1))
    for (i in 1:(n.levels+1)) {
      colors.rgb <- as.numeric(scan(text=urlcolors[i+2], what="",quiet=TRUE))
      colors[i] <- rgb(colors.rgb[1],colors.rgb[2],colors.rgb[3], maxColorValue=255)
    }
    out<-list(colors=colors, levels=levels)
  }
