#' Function to get a palette from the NCAR web site
#'
#' The functions retrieves a palette from the ETHZ web site
#'    http://www.ncl.ucar.edu/Document/Graphics/ColorTables/Files/
#'
#' @param palette.name  Name of the palette as listed on the NCAR web site
#' @param url url of the NCAR palette web site
#'
#' @return A list variable with
#' \item{colors}{The colours of the palette}
#' \item{levels}{An example of levels}
#' 
#' @author Burkhardt Rockel (HZG)
#' 
#' @export

get_ncar_palette <-
  function(palette.name, url="http://www.ncl.ucar.edu/Document/Graphics/ColorTables/Files/")
  {
    
    file<-paste(url,palette.name,".rgb",sep="")
    urlcolors<-readLines(file)
    urlcolors<-urlcolors[urlcolors!="" & !startsWith(urlcolors,'#')]
    n.colors<-as.numeric(strsplit(x = urlcolors[1], split = "=", fixed = T)[[1]][2])
    colors<-c(1:n.colors)
    for (i in 1:(n.colors)) {
      colors.rgb <- as.numeric(scan(text=urlcolors[i+1], what="",quiet=TRUE))
      colors[i] <- rgb(colors.rgb[1],colors.rgb[2],colors.rgb[3], maxColorValue=255)
    }
    out<-list(colors=colors)
  }
