% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frac_in_polygon.R
\name{frac_in_polygon}
\alias{frac_in_polygon}
\title{Function to compute what fraction of grid cells lies within  the given polygon}
\usage{
frac_in_polygon(lon, lat, poly.x, poly.y, multiply = 4)
}
\arguments{
\item{lon}{A vector of grid longitudes}

\item{lat}{A vector of grid latitudes}

\item{poly.x}{The longitudes of the edges of the polygon}

\item{poly.y}{The latitudes of the edges of the polygon}

\item{multiply}{Factor by which the grid is inflated for computation}
}
\value{
Returns a matrix of dimension nlon x nlat of fractional gridbox area within the polygon
}
\description{
This function computes for a regular grid (one vector of longitudes and latitudes), what fraction of each grid cell lies within the given polygon.
}
\details{
The accuracy for this method scales with the multiplying factor. After inflation, grid points within the polygon are found with \code{point.in.polygon} from the package \code{sp}
}
\examples{
   # set up latitudes and longitudes
   lon       <- seq(-3.5,3.5,0.5)
   lat       <- seq(-3.5,3.5,0.5)
   # polygon is a circle with radius 3
   poly.x    <- 3*sin(seq(0,2*pi,length=100))+0.13
   poly.y    <- 3*cos(seq(0,2*pi,length=100))-0.32
   outgrid   <- frac_in_polygon(lon,lat,poly.x,poly.y, multiply=5)
   # plot the result
   image(lon,lat,outgrid, asp=1)
   lines(poly.x, poly.y, lwd=2)

}
\seealso{
\code{\link[sp]{point.in.polygon}}
}
\author{
Jonas Bhend
}
