% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_colourbar.R
\name{plot_colourbar}
\alias{plot_colourbar}
\alias{plot_colourbar.default}
\alias{plot_colourbar.plotmap}
\title{Function to plot a colourbar}
\usage{
plot_colourbar(x, ...)

\method{plot_colourbar}{default}(levs, cols, side = 1, ylab = "",
  labels = NULL, xlab = "", nn = 1, center = F, cex.axis = 1,
  sea.col = NULL, sea.lab = NULL, border = par("fg"),
  triangle_ends = NULL, ...)

\method{plot_colourbar}{plotmap}(x, incl.units = T, side = 1,
  cex.axis = 1, labels = NULL, center = F, ...)
}
\arguments{
\item{levs}{Levels (boundaries)}

\item{cols}{colours to use}

\item{side}{At which side of the image the colour bar will be plottet: 1=bottom 2=left, 3=top, 4=right}

\item{labels}{If not NULL it overwrites the default labels given by levels in x.list}

\item{nn}{Each nn axis label will be drawn}

\item{center}{If TRUE the labels will be drawn in the center beside the colors}

\item{cex.axis}{The magnification to be used for axis annotation relative to the current setting of "cex""}

\item{border}{Colour of the border around the colour bar and the axis (but not ticks)}

\item{triangle_ends}{If not NULL it must be a logical vector with two elements. If an element is TRUE a triangle is plottet instead of a rectangle at the end of the color bar. First element gives the values for the start of the colour bar, the second element for the end of the colour bar}

\item{incl.units}{allows for the long-name and units attribute of the plotted variable to be added to the axis annotation)}

\item{x.list}{A list of class "plotmap" containing the levels and colours and additional information as returned by \code{plotmap}}
}
\value{
Nothing to return
}
\description{
Generic function to plot a horizontal or vertical colourbar with axis on the specified side.
}
\details{
There must be one more levels than colours including the colour for water (\code{sea.col}). 
   The enumeration of side is consistent with \code{axis()}, e.g. from 1 to 4 clockwise from bottom.
   Only every nn'th label is drawn. 
   Ticks are omitted if centered labels are drawn.
}
\section{Methods (by class)}{
\itemize{
\item \code{default}: Default, plots a colour bar given the levs and cols

\item \code{plotmap}: Color bar for plotmap
}}

\examples{
   tmp <- plotmap(file, ...)
   plot_colourbar(tmp, triangle_ends=c(TRUE,FALSE),border=NA)
   
}
\seealso{
\code{\link{plotmap}}
}
\author{
Jonas Bhend, Burkhardt Rockel (HZG)
}
