% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotmap.R
\name{plotmap}
\alias{plotmap}
\title{Plot NetCDF data on rotated, multiple rotated or regular lonlat grids}
\usage{
plotmap(file, sponge = 0, varname = NULL, lsm.file = NULL,
  units = NULL, longname = NULL, add.offset = 0, scale.factor = 1,
  col = NULL, levels = NULL, sea.col = NULL, sea.lab = "water",
  na.col = NULL, rivers = T, lakes = T, cities = T, label = TRUE,
  minpop = NULL, ncities = 10, city.pch = 19, alt.lev = NULL,
  grid = TRUE, grid.txt = c(T, T, T, T), grid.lty = 2,
  grid.lwd = 1, i.time = 1, i.lev = 1, map.lwd = 2, map.lty = 1,
  cex.axis = 1, cex.lab = 1, cex.main = 1, cex.txt = 1,
  main = "", xlab = "", ylab = "", add = FALSE,
  colourplot = TRUE, hires = FALSE, interior = FALSE,
  alt.poli = TRUE, nlongrid = 10, nlatgrid = 5, lon.ind, lat.ind,
  pollonshift = TRUE, smoothed = FALSE,
  geo.coordinate.names = c("lon", "lat"),alt.contour=FALS)
}
\arguments{
\item{file}{Path to the NetCDF file containing the data to plot}

\item{sponge}{Number of grid cells to mask out at boundary}

\item{varname}{Variable to plot}

\item{lsm.file}{Additional file containing land-sea mask and/or altitude 
information in quantities with the \code{standard_names}: 
\code{land_area_fraction} and \code{surface_altitude}}

\item{units}{overwrites the units from the netCDF file}

\item{longname}{overwrites the longname from the netCDF file}

\item{add.offset}{adds a constant to the quantity to be plotted}

\item{scale.factor}{multiplies the quantity to be plotted with a constant}

\item{col}{Colours to use}

\item{levels}{Levels to use}

\item{sea.col}{Colour for masked out water grid points}

\item{sea.lab}{Label for the water on the color bar}

\item{na.col}{Colour for grid points with undefined values}

\item{rivers}{Switch for automatically added rivers}

\item{lakes}{Switch for automatically added lakes}

\item{cities}{Switch for automatically added cities}

\item{label}{Switch for labels of cities}

\item{minpop}{Minimal population for cities to get drawn}

\item{ncities}{Number of cities to draw}

\item{city.pch}{Symbol for city points}

\item{alt.lev}{Level at which to draw coastline (if alt.contour == TRUE) in
units of \code{"FR_LAND"}}

\item{grid}{Switch for inclusion of grid lines}

\item{grid.txt}{Switch for inclusion of grid axis labels at the border of 
the plot. Can be set for each side separately by using grid.txt=c(T,T,T,T)}

\item{grid.lty}{Line type for grid lines}

\item{grid.lwd}{Line width for grid lines}

\item{i.time}{Time step to plot}

\item{i.lev}{Level to plot}

\item{map.lwd}{Line width for the coastline and country borders}

\item{map.lty}{Line type for the coastline and country borders}

\item{cex.axis}{Character expansion for axis lat/lon labels}

\item{cex.lab}{Character expansion for  x and y labels}

\item{cex.main}{Character expansion for main titles}

\item{cex.txt}{Character expansion for city labels}

\item{add}{Should plot be added to existing plot}

\item{colourplot}{Switch for colour or grayscale plot}

\item{hires}{Use hi-resolution coastline}

\item{interior}{Draw political boundaries}

\item{alt.poli}{Switch to use alternative political boundaries}

\item{nlongrid}{Approximate number of longitude grid lines}

\item{nlatgrid}{Approximate number of latitude grid lines}

\item{lon.ind}{Sequence of longitudes at which to 
draw grid lines. Has to be increasing values. Longitudes greater than 180 are 
converted.}

\item{lat.ind}{Sequence of latitudes at which to 
draw grid lines. Has to be increasing values.}

\item{pollonshift}{Effects only special cases where pollat=90. in rotated
coordinates. For example for rotated==geographic coordinate system 
pollon=0., pollat=90.. However, due to a bug in nearly all existing 
conversion routines, this is mostly defined as pollon=180., pollat=90. 
or pollon=0., pollat=90., polgam=180.. In the latter case set 
pollonshift=T}

\item{smoothed}{Smooth the plot. Internally use of \code{filled.contour} 
instead of \code{image}}

\item{geo.coordinate.names}{names fo the 2D fields of the geographical 
longitude and latitude at the model grid points}

\item{alt.contour}{Switch for usage of the coastline as in file or lsm.file}
}
\value{
A List of class "plotmap" containing the following objects:
\item{lev}{The levels used}
\item{col}{The colours used}
\item{sea.col}{The colour used for water grid points}
\item{sea.lab}{Label for the water on the color bar}
\item{flag_values}{Flag values taken from the netCDF attribute flag_values, if available}
\item{flag_meanings}{Flag meanings taken from the netCDF attribute flag_meanings, if available}
\item{longname}{The longname attribute of the variable plotted}
\item{units}{The units of the variable plotted}
\item{gridmapping}{A list containing the grid mapping attributes and additionals. For example rotated pole coodinates: gridmapping=list(attribute1=pollon, attribute2=pollat, attribute3=polgam, attribute4=pollonshift) }
}
\description{
This function reads in NetCDF data and produces a plot of the specified variable.
}
\note{
This function should works for geographical and rotated
    model coordinates and for most structured grids, if additional geographic lat/lon data are provided for the model grid.
    It is possible to plot icosahedral output. The geographical longitudes and latitudes of the vertices have to be given in variables with the names \code{clon_bnds} and \code{clat_bnds} or \code{clon_vertices} and \code{clat_vertices}.
}
\examples{
   tmp<-get.ethz.palette("kd_temp_1m")
   col<-tmp$colors
   levs<-tmp$levels
   out <- plotmap(...,col=col,levels=levs, ...)

}
