SUBROUTINE find_closest (undef, ptlon, ptlat, xvals, yvals, npt, nx, ny, xindex, yindex )

    IMPLICIT NONE

  INTEGER :: npt, nx, ny
  REAL *8 :: xvals(nx,ny), yvals(nx,ny), ptlon(npt), ptlat(npt), undef
  REAL *8 :: xindex(npt), yindex(npt)
  REAL :: dx, dy, dxm1, dxp1, dym1, dyp1, dist 
  REAL :: distx=0., disty=0., mulx=1., muly=1., xweight=0., yweight=0.
  REAL :: distadj(4), dist_locx_xnext=0., dist_locy_ynext=0.
  
  REAL :: haversine
  
  INTEGER :: idx, idy, iadj, idxm1, idxp1, idym1, idyp1
  INTEGER :: k
  REAL :: locx, locy
  REAL :: NaN=0.

  NaN = NaN / NaN

  DO k = 1, npt
  
    locx = REAL(ptlon(k))
    locy = REAL(ptlat(k))
    
    IF (locx == undef .OR. locy == undef) THEN
      xindex(k) = NaN
      yindex(k) = NaN
      CYCLE
    ENDIF
     
    CALL find_closest_pt (locx, locy, xvals, yvals, nx, ny, idx, idy )

    idx = idx + 1
    idy = idy + 1
    IF (idx < 1 .OR. idx > nx .OR. idy < 1 .OR. idy > ny) THEN  ! NaN IF
      xindex(k) = NaN
      yindex(k) = NaN
    ELSE
      idxp1 = MIN(nx,idx+1)
      idxm1 = MAX(1, idx-1)
      idyp1 = MIN(ny,idy+1)
      idym1 = MAX(1, idy-1)
      dx   = locx - REAL(xvals(idx,idy))
      dy   = locy - REAL(yvals(idx,idy))
      dxm1 = locx - REAL(xvals(idxm1,idy))
      dxp1 = locx - REAL(xvals(idxp1,idy))
      dym1 = locy - REAL(yvals(idx,idym1))
      dyp1 = locy - REAL(yvals(idx,idyp1))
       dist = haversine (locx, locy, REAL(xvals(idx,idy)), REAL(yvals(idx,idy)))
!... 1 = ll
!... 2 = ul
!... 3 = ur
!... 4 = lr
       distadj(1) = haversine (locx, locy, REAL(xvals(idxm1,idym1)), REAL(yvals(idxm1,idym1)))
       distadj(2) = haversine (locx, locy, REAL(xvals(idxm1,idyp1)), REAL(yvals(idxm1,idyp1)))
       distadj(3) = haversine (locx, locy, REAL(xvals(idxp1,idyp1)), REAL(yvals(idxp1,idyp1)))
       distadj(4) = haversine (locx, locy, REAL(xvals(idxp1,idym1)), REAL(yvals(idxp1,idym1)))
      iadj = MINLOC(distadj, DIM=1)
      SELECT CASE (iadj)
        CASE (1)
          distx = haversine (locx, locy, REAL(xvals(idxm1,idy)), REAL(yvals(idxm1,idy)))
          disty = haversine (locx, locy, REAL(xvals(idx,idym1)), REAL(yvals(idx,idym1)))
          dist_locx_xnext = haversine(REAL(xvals(idx,idy)), REAL(yvals(idx,idy)), REAL(xvals(idxm1,idy)), REAL(yvals(idxm1,idy)))
          dist_locy_ynext = haversine(REAL(xvals(idx,idy)), REAL(yvals(idx,idy)), REAL(xvals(idx,idym1)), REAL(yvals(idx,idym1)))
          mulx = -1.
          muly = -1.
        CASE (2)
          distx = haversine (locx, locy, REAL(xvals(idxm1,idy)), REAL(yvals(idxm1,idy)))
          disty = haversine (locx, locy, REAL(xvals(idx,idyp1)), REAL(yvals(idx,idyp1)))
          dist_locx_xnext = haversine(REAL(xvals(idx,idy)), REAL(yvals(idx,idy)), REAL(xvals(idxm1,idy)), REAL(yvals(idxm1,idy)))
          dist_locy_ynext = haversine(REAL(xvals(idx,idy)), REAL(yvals(idx,idy)), REAL(xvals(idx,idyp1)), REAL(yvals(idx,idyp1)))
          mulx = -1.
          muly = 1.
        CASE (3)
          distx = haversine (locx, locy, REAL(xvals(idxp1,idy)), REAL(yvals(idxp1,idy)))
          disty = haversine (locx, locy, REAL(xvals(idx,idyp1)), REAL(yvals(idx,idyp1)))
          dist_locx_xnext = haversine(REAL(xvals(idx,idy)), REAL(yvals(idx,idy)), REAL(xvals(idxp1,idy)), REAL(yvals(idxp1,idy)))
          dist_locy_ynext = haversine(REAL(xvals(idx,idy)), REAL(yvals(idx,idy)), REAL(xvals(idx,idyp1)), REAL(yvals(idx,idyp1)))
          mulx = 1.
          muly = 1.
        CASE (4)
          distx = haversine (locx, locy, REAL(xvals(idxp1,idy)), REAL(yvals(idxp1,idy)))
          disty = haversine (locx, locy, REAL(xvals(idx,idym1)), REAL(yvals(idx,idym1)))
          dist_locx_xnext = haversine(REAL(xvals(idx,idy)), REAL(yvals(idx,idy)), REAL(xvals(idxp1,idy)), REAL(yvals(idxp1,idy)))
          dist_locy_ynext = haversine(REAL(xvals(idx,idy)), REAL(yvals(idx,idy)), REAL(xvals(idx,idym1)), REAL(yvals(idx,idym1)))
          mulx = 1.
          muly = -1.
     END SELECT

      IF (dist_locy_ynext == 0.) THEN
        yweight = 0.
        IF (dist_locx_xnext == 0.) THEN
          xweight = 0.
        ELSE
         xweight = ABS(dist - distx)
        ENDIF
      ELSE
        IF (dist_locx_xnext == 0.) THEN
          xweight = ABS(dist - distx)
        ELSE
          yweight=(dist*dist + dist_locy_ynext*dist_locy_ynext - disty*disty) / (2. * dist_locy_ynext)
          xweight=SQRT(dist*dist - yweight*yweight)
        ENDIF
      ENDIF

      xindex(k)=idx + xweight/dist_locx_xnext * mulx
      yindex(k)=idy + yweight/dist_locy_ynext * muly
    ENDIF ! NaN IF

  ENDDO
  

END SUBROUTINE find_closest

SUBROUTINE find_closest_pt (locx, locy, xvals, yvals, nx, ny, idx, idy )

    IMPLICIT NONE

  INTEGER :: nx, ny
  REAL :: locx, locy
  REAL *8 :: xvals(nx,ny), yvals(nx,ny)
  INTEGER :: idx, idy
  
  INTEGER :: init_guess_idx, init_guess_idy
  INTEGER :: last_idx=1, last_idy=1
  LOGICAL :: have_been_here_before=.FALSE.
 
    IF (have_been_here_before) THEN
      init_guess_idx = last_idx
      init_guess_idy = last_idy
    ELSE
      init_guess_idx = nx/2
      init_guess_idy = ny/2
    ENDIF
      
    CALL find_closest_pt_inner ( init_guess_idx, init_guess_idy, locx, locy, xvals, yvals, nx, ny, idx, idy )

    have_been_here_before=.TRUE.
    IF (idx == -999) THEN
      last_idx = nx/2
      last_idy = ny/2
    ELSE
      last_idx = idx
      last_idy = idy
    ENDIF

END SUBROUTINE find_closest_pt

SUBROUTINE find_closest_pt_inner ( init_guess_idxx, init_guess_idxy, locx, locy, xvals, yvals, nx, ny, idxx, idxy )

    IMPLICIT NONE

    INTEGER :: init_guess_idxx, init_guess_idxy
    REAL :: locx, locy
    REAL *8 :: xvals(nx,ny), yvals(nx,ny)
    INTEGER, INTENT (IN) :: nx, ny
    INTEGER :: idxx, idxy
  
    INTEGER :: i, j
    INTEGER :: index, i2use, j2use, ox, oy, idx_dest, idx_src, isrc, jsrc
    INTEGER :: nsteps, curx, cury, minloc
    INTEGER :: n_wrapped_x, n_wrapped_y
    LOGICAL :: have_calc(0:8)
    REAL :: mindist, dist(0:8), tdist(0:8), prev_d4
!    REAL    :: dx, dy

    REAL :: haversine

    nsteps = 0
    n_wrapped_x = 0
    n_wrapped_y = 0

    curx = init_guess_idxx
    cury = init_guess_idxy

    have_calc(:) = .FALSE.

    minloc = -1
    DO WHILE ( minloc /= 4 ) 
        nsteps = nsteps + 1
        mindist = 1.E31
        DO j = -1,1
        DO i = -1,1
            index = (i+1) + (-j+1)*3
            IF ( .NOT. have_calc(index) ) THEN

                IF ( (curx == 0) .AND. (i == -1)) THEN
                    i2use = nx-1
!                    i2use = 0
                ELSE IF( (curx == nx-1) .AND. (i == 1)) THEN
                    i2use = 0
!                    i2use = nx-1
                ELSE
                    i2use = curx + i
                ENDIF

                IF( (cury == 0) .AND. (j == -1)) THEN
                    j2use = ny-1
!                    j2use = 0
                ELSE IF( (cury == ny-1) .AND. (j == 1)) THEN
                    j2use = 0
!                    j2use = ny - 1
                ELSE
                    j2use = cury + j
                ENDIF
                
!                 dx = locx - REAL(xvals(i2use+1, j2use+1))
!                 IF( dx /= dx ) THEN
!                     WRITE(*,*) "Error, bad dx locx=",locx,"  i2use,j2use=",i2use,j2use,"  xvals[i2use,j2use]=",xvals(i2use, j2use)
!                     STOP (-1)
!                 ENDIF
! 
!                 dy = locy - REAL(yvals(i2use+1, j2use+1))
!                 dist(index) = dx*dx + dy*dy    ! don't bother with square root, takes time and won't affect minimum finding */
                dist(index) = haversine (locx, locy, REAL(xvals(i2use+1, j2use+1)), REAL(yvals(i2use+1, j2use+1)))

            ENDIF

            IF( dist(index) < mindist ) THEN
                mindist = dist(index)
                minloc  = index
            ENDIF
            
        ENDDO ! i loop
        ENDDO ! j loop
        
        prev_d4 = dist(4)
        
        ! Now do the shifting towards the downhill direction 
        !
        !  0 1 2
        !  3 4 5
        !  6 7 8
        !    |
        !    |
        !  (0,0) -----
        !

        have_calc(:) = .FALSE.

        SELECT CASE( minloc )
            CASE (0)
                ox=-1; oy=1
            CASE (1)
                ox=0;  oy=1
            CASE (2)
                ox=1;  oy=1
            CASE (3)
                ox=-1; oy=0
            CASE (4)
                ox=0;  oy=0
            CASE (5)
                ox=1;  oy=0
            CASE (6)
                ox=-1; oy=-1
            CASE (7)
                ox=0;  oy=-1
            CASE (8)
                ox=1;  oy=-1
            CASE DEFAULT
                WRITE(*,*) "coding error in find_closest.f90, minloc=", minloc
                STOP (-1);
        END SELECT

        !... Update current x,y position 
        IF ( (curx == 0) .AND. (ox == -1)) THEN
            n_wrapped_x = n_wrapped_x + 1
            curx = nx-1
        ELSE IF( (curx == nx-1) .AND. (ox == 1)) THEN
            n_wrapped_x = n_wrapped_x + 1
            curx = 0
        ELSE
            curx = curx + ox
        ENDIF

        IF ( (cury == 0) .AND. (oy == -1)) THEN
            n_wrapped_y = n_wrapped_y + 1
            cury = ny-1
        ELSE IF( (cury == ny-1) .AND. (oy == 1)) THEN
            n_wrapped_y = n_wrapped_y + 1
            cury = 0
        ELSE
            cury = cury + oy
        ENDIF

        IF ( (curx<0) .OR. (cury<0) .OR. (curx>=nx) .OR. (cury>=ny)) THEN
            WRITE(*,*) "Error, alg fails, cursor off array; cursor=(",curx,",",cury,")  nx=",nx,"  ny=", ny
            STOP (-1)
        ENDIF

        IF ( (n_wrapped_x > 2) .OR. (n_wrapped_y > 2)) THEN
            !... Give up
            idxx = -999
            idxy = -999
            write(*,*) 'Gave up ',locx, locy, curx, cury
!            idxx = curx
!            idxy = cury
            RETURN
        ENDIF

        !... Shift the distance array appropriately
        DO  i = -1, 1
        DO  j = -1, 1
            idx_dest = (i+1) + (-j+1)*3
            isrc = i + ox
            jsrc = j + oy
            IF ( (isrc>=-1) .AND. (isrc<=1) .AND. (jsrc>=-1) .AND. (jsrc<=1)) THEN
                idx_src = (isrc+1) + (-jsrc+1)*3
                IF( (idx_src<0) .OR. (idx_src>8)) THEN
                    WRITE(*,*) "Error, alg failed! overlay.c while shifting"
                    STOP (-1)
                ENDIF
                tdist(idx_dest) = dist(idx_src)
                have_calc(idx_dest) = .TRUE.
            ENDIF
        ENDDO
        ENDDO

        dist(:) = tdist(:)

        IF ( dist(4) > prev_d4 ) THEN
            WRITE(*,*) "error, alg fails; prev center dist=",prev_d4,"  new=",dist(4),"  should always be getting closer!"
            STOP (-1);
        ENDIF

    ENDDO ! minloc while

    idxx = curx
    idxy = cury

END SUBROUTINE find_closest_pt_inner

REAL FUNCTION haversine (lon1, lat1, lon2, lat2)
  IMPLICIT NONE

  REAL :: lon1, lat1, lon2, lat2
  
  REAL :: R=6371
  REAL :: deg2rad=0.0174532925
  REAL :: radlat1, radlat2, dradlon, dradlat, sindradlat, sindradlon
  REAL :: a, c

  radlat1 = lat1 * deg2rad
  radlat2 = lat2 * deg2rad
  dradlat = (lat2-lat1) * deg2rad
  dradlon = (lon2-lon1) * deg2rad
  sindradlat=SIN(dradlat/2)
  sindradlon=SIN(dradlon/2)
  a = sindradlat * sindradlat + COS(radlat1) * COS(radlat2) * sindradlon * sindradlon
  c = 2 * ATAN2(SQRT(a), SQRT(1-a))
  haversine = R * c
END FUNCTION haversine
